
(* closure idioms *)

let addn m n = m + n

let add_one m = addn 1

let add_two m = addn 2

let rec f m = 
  if m=0 
  then [] 
  else (addn m)::(f (m-1))

let lst65432 = List.map (fun x -> x 1) (f 5) 

let f1 g h = (fun x -> g (h x)) (* call this compose *)

(* already provided in Pervasives (no module name needed *)
type 'a option = None | Some of 'a 

let f2 g h x = 
  match g x with
    None   -> h x
  | Some y -> y

(* just a function pointer *)
let print_int = f1 print_string string_of_int
(* a closure *)
let truncate1 (lim : int) f = f1 (fun x -> min lim x) f
let truncate2 (lim : int) f = f1 (min lim) f
 (* weird typing note: min <, >=, etc. happen to work on anything, but
    this is a hack (should be int *)

(* already provided as List.map *)
let rec map f lst =
  match lst with
    []     -> []
  | hd::tl -> (f hd)::(map f tl)

(* just a function pointer *)
let incr lst = map (fun x -> x+1) lst
let incr = map (fun x -> x+1)
(* a closure *)
let mul i lst = map (fun x -> x*i) lst
let mul i = map (fun x -> x*i)

(* already provided as List.fold_left *)
let rec fold_left f acc lst =
  match lst with
    []     -> acc
  | hd::tl -> fold_left f (f acc hd) tl

(* just function pointers *)
let f1 = fold_left (fun x y -> x+y) 0
let f2 = fold_left (fun x y -> x && y>0) true
(* a closure *)
let f1 lst lo hi = 
 fold_left 
   (fun x y -> if y>lo && y<hi then x+1 else x)
   0
   lst

(* fancier than what most people do, but worth seeing... *)

type set = { add : int -> set; member : int -> bool }

let empty_set = 
  let exists lst j = (*could use List.exists or List.fold_left *)
     let rec iter rest =
        match rest with 
          [] -> false 
        | hd::tl -> j=hd || iter tl in
     iter lst in
  let rec make_set lst = 
     { add    = (fun i -> make_set(i::lst));
       member = exists lst } in
  make_set []

(* tail recursion examples *)

(* not tail recursive *)
let rec fact1 x = 
  if x==0 then 1 else x * (fact1(x-1))

(* tail recursive *)
let fact2 x = 
   let rec f acc x =
   if x==0 then acc else f (acc*x) (x-1)
   in
   f 1 x

(* not tail recursive *)
let rec sum1 lst = 
  match lst with 
    []     -> 0
  | hd::tl -> hd + (sum1 tl)

(* tail recursive *)
let sum2 lst = 
  let rec f acc lst = 
     match lst with 
      []     -> acc
    | hd::tl -> f (acc+hd) tl
  in
  f 0 lst

(* one tail-recurisve call, one not (no point in avoiding that here) *)
type tree = Leaf of int | Node of tree * tree
let sum tr = 
  let rec f acc tr = 
     match tr with 
      Leaf i -> acc+i
    | Node(left,right) -> f (f acc left) right
  in
  f 0 tr
